﻿using System;
using System.Collections.ObjectModel;
using System.Net;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Ioc;
using GalaSoft.MvvmLight.Messaging;
using Microsoft.Practices.ServiceLocation;
using RssReader.Helpers;
using RssReader.Model;

namespace RssReader.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        private const string DetailsPageUrl = "/DetailsPage.xaml?item={0}";

        private readonly INavigationService _navigationService;
        private readonly IRssService _rssService;
        private RelayCommand<RssArticle> _navigateToArticleCommand;
        private RelayCommand _refreshCommand;

        public IDialogService DialogService
        {
            get
            {
                return ServiceLocator.Current.GetInstance<IDialogService>();
            }
        }

        public ObservableCollection<RssArticle> Items
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the NavigateToArticleCommand.
        /// </summary>
        public RelayCommand<RssArticle> NavigateToArticleCommand
        {
            get
            {
                return _navigateToArticleCommand
                    ?? (_navigateToArticleCommand = new RelayCommand<RssArticle>(
                        article =>
                        {
#if NETFX_CORE
                            _navigationService.NavigateTo(typeof (DetailsPage), article);
#else
                            var url = HttpUtility.UrlEncode(article.Link.AbsoluteUri);
                            SimpleIoc.Default.Register(() => article, url);
                            _navigationService.NavigateTo(
                                new Uri(string.Format(DetailsPageUrl, url), UriKind.Relative));
#endif
                        }));
            }
        }

        /// <summary>
        /// Gets the RefreshCommand.
        /// </summary>
        public RelayCommand RefreshCommand
        {
            get
            {
                return _refreshCommand
                    ?? (_refreshCommand = new RelayCommand(
                        async () =>
                        {
                            await Refresh();
                        }));
            }
        }

        public MainViewModel(
            IRssService rssService,
            INavigationService navigationService)
        {
            _rssService = rssService;
            _navigationService = navigationService;
            Items = new ObservableCollection<RssArticle>();

#if DEBUG
            if (IsInDesignMode)
            {
                Refresh();
            }
#endif
        }

        public async Task Refresh()
        {
            Items.Clear();

            try
            {
                Messenger.Default.Send(new StatusMessage("Getting articles", 0));

                var list = await _rssService.GetArticles();

                if (list.Count == 0)
                {
                    DialogService.ShowMessage(
                        "We couldn't find any articles",
                        "Nothing found",
                        "Too bad!");
                }

                foreach (var item in list)
                {
                    Items.Add(item);
                }

                Messenger.Default.Send(new StatusMessage("Done", 3000));
            }
            catch (Exception ex)
            {
                // Hide status
                Messenger.Default.Send(new StatusMessage(string.Empty, 1));

                DialogService.ShowError(
                    ex,
                    "Error when loading",
                    "Oops");
            }
        }
    }
}